@@ -43,6 +43,17 @@
 
 
 
+void
+_gpgme_wipememory (void *ptr, size_t len)
+{
+  /* Prevent compiler from optimizing away the call to memset by accessing
+   * memset through volatile pointer. */
+  static void *(*volatile memset_ptr)(void *, int, size_t) = (void *)memset;
+  memset_ptr (ptr, 0, len);
+}
+
+
+
 static char *
 do_strconcat (const char *s1, va_list arg_ptr)
 {
@@ -423,10 +434,10 @@ _gpgme_strtoul_field (const char *string, unsigned long *result)
 /* Convert STRING into an offset value.  Note that this functions only
  * allows for a base-10 length.  This function is similar to atoi()
  * and thus there is no error checking.  */
-gpgme_off_t
+uint64_t
 _gpgme_string_to_off (const char *string)
 {
-  gpgme_off_t value = 0;
+  uint64_t value = 0;
 
   while (*string == ' ' || *string == '\t')
     string++;
