@@ -1302,11 +1302,9 @@ _ntbtls_ciphersuite_list (void)
         {
           if ((suite = _ntbtls_ciphersuite_from_id (ciphersuite_preference[i])))
             {
-              /*FIXME: GCM and CCM are not yet ready for us - disable.  */
-              if (suite->ciphermode != GCRY_CIPHER_MODE_GCM
-                  && suite->ciphermode != GCRY_CIPHER_MODE_CCM
+              /*FIXME: CCM are not yet ready for us - disable.  */
+              if (suite->ciphermode != GCRY_CIPHER_MODE_CCM
                   && suite->key_exchange != KEY_EXCHANGE_ECDH_RSA
-                  && suite->key_exchange != KEY_EXCHANGE_ECDHE_ECDSA
                   && suite->key_exchange != KEY_EXCHANGE_ECDH_ECDSA)
                 supported_ciphersuites[j++] = ciphersuite_preference[i];
             }
