@@ -71,6 +71,11 @@
 #include "../common/zb32.h"
 
 
+/* FIXME: Libgcrypt 1.9 will support EAX.  Until we name this a
+ * requirement we hardwire the enum used for EAX.  */
+#define MY_GCRY_CIPHER_MODE_EAX 14
+
+
 #ifdef ENABLE_SELINUX_HACKS
 /* A object and a global variable to keep track of files marked as
    secured. */
@@ -98,7 +103,7 @@ register_secured_file (const char *fname)
   struct secured_file_item *sf;
 
   /* Note that we stop immediately if something goes wrong here. */
-  if (stat (fname, &buf))
+  if (gnupg_stat (fname, &buf))
     log_fatal (_("fstat of '%s' failed in %s: %s\n"), fname,
                "register_secured_file", strerror (errno));
 /*   log_debug ("registering '%s' i=%lu.%lu\n", fname, */
@@ -127,7 +132,7 @@ unregister_secured_file (const char *fname)
   struct stat buf;
   struct secured_file_item *sf, *sfprev;
 
-  if (stat (fname, &buf))
+  if (gnupg_stat (fname, &buf))
     {
       log_error (_("fstat of '%s' failed in %s: %s\n"), fname,
                  "unregister_secured_file", strerror (errno));
@@ -201,7 +206,7 @@ is_secured_filename (const char *fname)
 
   /* Note that we print out a error here and claim that a file is
      secure if something went wrong. */
-  if (stat (fname, &buf))
+  if (gnupg_stat (fname, &buf))
     {
       if (errno == ENOENT || errno == EPERM || errno == EACCES)
         return 0;
@@ -236,7 +241,7 @@ checksum_u16( unsigned n )
 
 
 u16
-checksum( byte *p, unsigned n )
+checksum (const byte *p, unsigned n)
 {
     u16 a;
 
@@ -252,6 +257,25 @@ checksum_mpi (gcry_mpi_t a)
   byte *buffer;
   size_t nbytes;
 
+  /*
+   * This code can be skipped when gcry_mpi_print
+   * supports opaque MPI.
+   */
+  if (gcry_mpi_get_flag (a, GCRYMPI_FLAG_OPAQUE))
+    {
+      const byte *p;
+      unsigned int nbits;
+
+      p = gcry_mpi_get_opaque (a, &nbits);
+      if (!p)
+        return 0;
+
+      csum = nbits >> 8;
+      csum += (nbits & 0xff);
+      csum += checksum (p, (nbits+7)/8);
+      return csum;
+    }
+
   if ( gcry_mpi_print (GCRYMPI_FMT_PGP, NULL, 0, &nbytes, a) )
     BUG ();
   /* Fixme: For numbers not in secure memory we should use a stack
@@ -307,12 +331,11 @@ print_cipher_algo_note (cipher_algo_t algo)
 void
 print_digest_algo_note (digest_algo_t algo)
 {
-  const enum gcry_md_algos galgo = map_md_openpgp_to_gcry (algo);
-  const struct weakhash *weak;
-
   if(algo >= 100 && algo <= 110)
     {
       static int warn=0;
+      const enum gcry_md_algos galgo = map_md_openpgp_to_gcry (algo);
+
       if(!warn)
 	{
 	  warn=1;
@@ -321,14 +344,13 @@ print_digest_algo_note (digest_algo_t algo)
                     gcry_md_algo_name (galgo));
 	}
     }
-  else
-      for (weak = opt.weak_digests; weak != NULL; weak = weak->next)
-        if (weak->algo == galgo)
-          {
-            es_fflush (es_stdout);
-            log_info (_("WARNING: digest algorithm %s is deprecated\n"),
-                      gcry_md_algo_name (galgo));
-          }
+  else if (is_weak_digest (algo))
+    {
+      const enum gcry_md_algos galgo = map_md_openpgp_to_gcry (algo);
+      es_fflush (es_stdout);
+      log_info (_("WARNING: digest algorithm %s is deprecated\n"),
+                gcry_md_algo_name (galgo));
+    }
 }
 
 
@@ -337,6 +359,10 @@ print_digest_rejected_note (enum gcry_md_algos algo)
 {
   struct weakhash* weak;
   int show = 1;
+
+  if (opt.quiet)
+    return;
+
   for (weak = opt.weak_digests; weak; weak = weak->next)
     if (weak->algo == algo)
       {
@@ -357,6 +383,25 @@ print_digest_rejected_note (enum gcry_md_algos algo)
 }
 
 
+void
+print_sha1_keysig_rejected_note (void)
+{
+  static int shown;
+
+  if (shown || opt.quiet)
+    return;
+
+  shown = 1;
+  es_fflush (es_stdout);
+  log_info (_("Note: third-party key signatures using"
+              " the %s algorithm are rejected\n"),
+            gcry_md_algo_name (GCRY_MD_SHA1));
+  if (!opt.quiet)
+    log_info (_("(use option \"%s\" to override)\n"),
+              "--allow-weak-key-signatures");
+}
+
+
 /* Print a message
  *  "(reported error: %s)\n
  * in verbose mode to further explain an error.  If the error code has
@@ -397,7 +442,7 @@ print_further_info (const char *format, ...)
 
   log_info (_("(further info: "));
   va_start (arg_ptr, format);
-  log_logv (GPGRT_LOG_CONT, format, arg_ptr);
+  log_logv (GPGRT_LOGLVL_CONT, format, arg_ptr);
   va_end (arg_ptr);
   log_printf (")\n");
 }
@@ -500,20 +545,6 @@ map_cipher_gcry_to_openpgp (enum gcry_cipher_algos algo)
     }
 }
 
-/* Map Gcrypt public key algorithm numbers to those used by OpenPGP.
-   FIXME: This mapping is used at only two places - we should get rid
-   of it.  */
-pubkey_algo_t
-map_pk_gcry_to_openpgp (enum gcry_pk_algos algo)
-{
-  switch (algo)
-    {
-    case GCRY_PK_ECDSA:  return PUBKEY_ALGO_ECDSA;
-    case GCRY_PK_ECDH:   return PUBKEY_ALGO_ECDH;
-    default: return algo < 110 ? (pubkey_algo_t)algo : 0;
-    }
-}
-
 
 /* Return the block length of an OpenPGP cipher algorithm.  */
 int
@@ -544,7 +575,7 @@ openpgp_cipher_blocklen (cipher_algo_t algo)
 
 /****************
  * Wrapper around the libgcrypt function with additional checks on
- * the OpenPGP contraints for the algo ID.
+ * the OpenPGP constraints for the algo ID.
  */
 int
 openpgp_cipher_test_algo (cipher_algo_t algo)
@@ -583,6 +614,87 @@ openpgp_cipher_algo_name (cipher_algo_t algo)
 }
 
 
+/* Same as openpgp_cipher_algo_name but returns a string in the form
+ * "ALGO.MODE".  If AEAD is 0 "CFB" is used for the mode.  */
+const char *
+openpgp_cipher_algo_mode_name (cipher_algo_t algo, aead_algo_t aead)
+{
+  return map_static_strings ("openpgp_cipher_algo_mode_name", algo, aead,
+                             openpgp_cipher_algo_name (algo),
+                             ".",
+                             aead? openpgp_aead_algo_name (aead) : "CFB",
+                             NULL);
+}
+
+
+/* Return 0 if ALGO is supported.  Return an error if not. */
+gpg_error_t
+openpgp_aead_test_algo (aead_algo_t algo)
+{
+  /* FIXME: We currently have no easy way to test whether libgcrypt
+   * implements a mode.  The only way we can do this is to open a
+   * cipher context with that mode and close it immediately.  That is
+   * a bit costly.  Thus in case we add another algo we need to look
+   * at the libgcrypt version and assume nothing has been patched out.  */
+  switch (algo)
+    {
+    case AEAD_ALGO_NONE:
+      break;
+
+    case AEAD_ALGO_EAX:
+    case AEAD_ALGO_OCB:
+      return 0;
+    }
+
+  return gpg_error (GPG_ERR_INV_CIPHER_MODE);
+}
+
+
+/* Map the OpenPGP AEAD algorithm with ID ALGO to a string
+ * representation of the algorithm name.  For unknown algorithm IDs
+ * this function returns "?".  */
+const char *
+openpgp_aead_algo_name (aead_algo_t algo)
+{
+  switch (algo)
+    {
+    case AEAD_ALGO_NONE:  break;
+    case AEAD_ALGO_EAX:   return "EAX";
+    case AEAD_ALGO_OCB:   return "OCB";
+    }
+
+  return "?";
+}
+
+
+/* Return information for the AEAD algorithm ALGO.  The corresponding
+ * Libgcrypt ciphermode is stored at R_MODE and the required number of
+ * octets for the nonce at R_NONCELEN.  On error and error code is
+ * returned.  Note that the taglen is always 128 bits.  */
+gpg_error_t
+openpgp_aead_algo_info (aead_algo_t algo, enum gcry_cipher_modes *r_mode,
+                        unsigned int *r_noncelen)
+{
+  switch (algo)
+    {
+    case AEAD_ALGO_OCB:
+      *r_mode = GCRY_CIPHER_MODE_OCB;
+      *r_noncelen = 15;
+      break;
+
+    case AEAD_ALGO_EAX:
+      *r_mode = MY_GCRY_CIPHER_MODE_EAX;
+      *r_noncelen = 16;
+      break;
+
+    default:
+      log_error ("unsupported AEAD algo %d\n", algo);
+      return gpg_error (GPG_ERR_INV_CIPHER_MODE);
+    }
+  return 0;
+}
+
+
 /* Return 0 if ALGO is a supported OpenPGP public key algorithm.  */
 int
 openpgp_pk_test_algo (pubkey_algo_t algo)
@@ -633,7 +745,7 @@ openpgp_pk_test_algo2 (pubkey_algo_t algo, unsigned int use)
 #endif
 
     case PUBKEY_ALGO_ELGAMAL:
-      /* Dont't allow type 20 keys unless in rfc2440 mode.  */
+      /* Don't allow type 20 keys unless in rfc2440 mode.  */
       if (RFC2440)
         ga = GCRY_PK_ELG;
       break;
@@ -644,6 +756,13 @@ openpgp_pk_test_algo2 (pubkey_algo_t algo, unsigned int use)
   if (!ga)
     return gpg_error (GPG_ERR_PUBKEY_ALGO);
 
+  /* Elgamal in OpenPGP used to support signing and Libgcrypt still
+   * does.  However, we removed the signing capability from gpg ages
+   * ago.  This function should reflect this so that errors are thrown
+   * early and not only when we try to sign using Elgamal.  */
+  if (ga == GCRY_PK_ELG && (use & (PUBKEY_USAGE_CERT | PUBKEY_USAGE_SIG)))
+    return gpg_error (GPG_ERR_WRONG_PUBKEY_ALGO);
+
   /* Now check whether Libgcrypt has support for the algorithm.  */
   return gcry_pk_algo_info (ga, GCRYCTL_TEST_ALGO, NULL, &use_buf);
 }
@@ -658,21 +777,21 @@ openpgp_pk_algo_usage ( int algo )
     switch ( algo ) {
       case PUBKEY_ALGO_RSA:
           use = (PUBKEY_USAGE_CERT | PUBKEY_USAGE_SIG
-                 | PUBKEY_USAGE_ENC | PUBKEY_USAGE_AUTH);
+                 | PUBKEY_USAGE_ENC | PUBKEY_USAGE_RENC | PUBKEY_USAGE_AUTH);
           break;
       case PUBKEY_ALGO_RSA_E:
       case PUBKEY_ALGO_ECDH:
-          use = PUBKEY_USAGE_ENC;
+          use = PUBKEY_USAGE_ENC | PUBKEY_USAGE_RENC;
           break;
       case PUBKEY_ALGO_RSA_S:
           use = PUBKEY_USAGE_CERT | PUBKEY_USAGE_SIG;
           break;
       case PUBKEY_ALGO_ELGAMAL:
           if (RFC2440)
-             use = PUBKEY_USAGE_ENC;
+             use = PUBKEY_USAGE_ENC | PUBKEY_USAGE_RENC;
           break;
       case PUBKEY_ALGO_ELGAMAL_E:
-          use = PUBKEY_USAGE_ENC;
+          use = PUBKEY_USAGE_ENC | PUBKEY_USAGE_RENC;
           break;
       case PUBKEY_ALGO_DSA:
           use = PUBKEY_USAGE_CERT | PUBKEY_USAGE_SIG | PUBKEY_USAGE_AUTH;
@@ -808,13 +927,18 @@ get_signature_count (PKT_public_key *pk)
 /* Expand %-strings.  Returns a string which must be xfreed.  Returns
    NULL if the string cannot be expanded (too large). */
 char *
-pct_expando(const char *string,struct expando_args *args)
+pct_expando (ctrl_t ctrl, const char *string,struct expando_args *args)
 {
   const char *ch=string;
   int idx=0,maxlen=0,done=0;
   u32 pk_keyid[2]={0,0},sk_keyid[2]={0,0};
   char *ret=NULL;
 
+  /* The parser below would return NULL for an empty string, thus we
+   * catch it here.  Also catch NULL here. */
+  if (!string || !*string)
+    return xstrdup ("");
+
   if(args->pk)
     keyid_from_pk(args->pk,pk_keyid);
 
@@ -930,7 +1054,7 @@ pct_expando(const char *string,struct expando_args *args)
 			PKT_public_key *pk=
 			  xmalloc_clear(sizeof(PKT_public_key));
 
-			if (!get_pubkey_fast (pk,args->pksk->main_keyid))
+			if (!get_pubkey_fast (ctrl, pk,args->pksk->main_keyid))
 			  fingerprint_from_pk (pk, array, &len);
 			else
 			  memset (array, 0, (len=MAX_FINGERPRINT_LEN));
@@ -1113,6 +1237,39 @@ string_to_cipher_algo (const char *string)
   return val;
 }
 
+
+/*
+ * Map an AEAD mode string to a an AEAD algorithm number as defined by
+ * rfc4880bis.  Also support the "An" syntax as used by the preference
+ * strings.
+ */
+aead_algo_t
+string_to_aead_algo (const char *string)
+{
+  int result;
+
+  if (!string)
+    result = 0;
+  else if (!ascii_strcasecmp (string, "EAX"))
+    result = 1;
+  else if (!ascii_strcasecmp (string, "OCB"))
+    result = 2;
+  else if ((string[0]=='A' || string[0]=='a'))
+    {
+      char *endptr;
+
+      string++;
+      result = strtol (string, &endptr, 10);
+      if (!*string || *endptr || result < 1 || result > 2)
+        result = 0;
+    }
+  else
+    result = 0;
+
+  return result;
+}
+
+
 /*
  * Wrapper around gcry_md_map_name to provide a fallback using the
  * "Hn" syntax as used by the preference strings.
@@ -1229,6 +1386,7 @@ default_cipher_algo(void)
     return opt.s2k_cipher_algo;
 }
 
+
 /* There is no default_digest_algo function, but see
    sign.c:hash_for() */
 
@@ -1263,10 +1421,6 @@ compliance_failure(void)
       ver="OpenPGP (older)";
       break;
 
-    case CO_PGP6:
-      ver="PGP 6.x";
-      break;
-
     case CO_PGP7:
       ver="PGP 7.x";
       break;
@@ -1401,15 +1555,18 @@ optlen(const char *s)
     return strlen(s);
 }
 
+
+/* Note: This function returns true on success.  */
 int
 parse_options(char *str,unsigned int *options,
 	      struct parse_options *opts,int noisy)
 {
   char *tok;
 
-  if (str && !strcmp (str, "help"))
+  if (str && (!strcmp (str, "help") || !strcmp (str, "full-help")))
     {
       int i,maxlen=0;
+      int full = *str == 'f';
 
       /* Figure out the longest option name so we can line these up
 	 neatly. */
@@ -1421,6 +1578,10 @@ parse_options(char *str,unsigned int *options,
         if(opts[i].help)
 	  es_printf("%s%*s%s\n",opts[i].name,
                     maxlen+2-(int)strlen(opts[i].name),"",_(opts[i].help));
+      if (full)
+        for (i=0; opts[i].name; i++)
+          if(!opts[i].help)
+            es_printf("%s\n",opts[i].name);
 
       g10_exit(0);
     }
@@ -1756,3 +1917,17 @@ additional_weak_digest (const char* digestname)
   weak->next = opt.weak_digests;
   opt.weak_digests = weak;
 }
+
+
+/* Return true if ALGO is in the list of weak digests.  */
+int
+is_weak_digest (digest_algo_t algo)
+{
+  const enum gcry_md_algos galgo = map_md_openpgp_to_gcry (algo);
+  const struct weakhash *weak;
+
+  for (weak = opt.weak_digests; weak; weak = weak->next)
+    if (weak->algo == galgo)
+      return 1;
+  return 0;
+}
